/**
 *  @file Vector.h
 *  @brief a vektor osztályunk interfésze
 */

#ifndef VECTOR_H
#define VECTOR_H

/** @brief ezt a makrót definiálva megjelennek a függvényekben a kiírások */
#define VECTOR_DEBUG

#include <iostream>

/**
 *  @brief vektor osztály
 *  @details
 *  Dinamikusan tároló, de fix méretű vektort valósítunk meg.
 *  Mindkét adattagunk konstans, ezért ezek csak az inicalizáló listán 
 *  inicializálhatóak. A méret tárolásához a size_t típust használjuk.
 *  
 *  Dinamikusan tároló osztálynál legtöbb esetben nem jók a default 
 *  tagfüggvények, meg kell valósítanunk a (1) másoló konstruktort, 
 *  (2) értékadás operátort, (3) destruktort. Általános szabály, hogy ha e három
 *  közül valamelyiket meg kell írnunk, szükséges a másik kettő megírása is. Ezt
 *  szokták 3-as szabálynak ("rule of three") is hívni a C++-ban.
 *  
 *  Az itt szereplő tagfüggvényeken kívül természetesen még rengeteg funkció
 *  megvalósítható, itt most az áttekinthetőség kedvéért ennyire szorítkozunk.
 */
class Vector 
{
	/** @brief a vektor elemeinek száma (hossza) */
	size_t const len;
	/** @brief mutató az elemekre */
	double * const data;
public:
	/**
	 *  @brief konstruktor
	 *  @param[in] len vektor elemszáma
	 *  @param[in] val elemek értéke
	 *  @details
	 *  Kétféle hívás lehetséges:
	 *  Vector v1(10, 1.3);		// Tíz elemű, mindegyik 1.3
	 *  Vector v2(8);			// Nyolc elemű, mindegyik 0.0
	 */
	Vector(size_t len, double val = 0.0);
	
	/**
	 *  @brief másoló konstruktor (copy ctor)
	 *  @param[in] v ezt másoljuk
	 */
	Vector(Vector const &v);
	
	/**
	 *  @brief értékadás (assignment) operátor
	 */
	Vector &operator=(Vector const &v);
	
	/**
	 *  @brief destruktor
	 */
	~Vector();
	
	/**
	 *  @brief getter a méret lekérdezéséhez
	 */
	size_t getLen() const 
	{
		return len;
	}
	
	/**
	 *  @brief indexelő operátor
	 *  @param[in] idx index
	 *  @details
	 *  Az indexelés eredménye nem állhat kifejezés bal oldalán ebben az 
	 *  esetben, vagyis az eredmény jobbérték. Ezért konstans referenciát ad 
	 *  vissza. Ez hívható konstans vektorra is.	 
	 */
	double const &operator[](size_t idx) const;
	
	/**
	 *  @brief indexelő operátor
	 *  @param[in] idx index
	 *  Az indexelés eredménye szerepelhet kifejezés bal oldalán, így 
	 *  változtatható pl. a vektor egy elemének értéke. Ezért konstans vektorra 
	 *  nem hívható ez a függvény.
	 *  Mi alapján különbözteti meg a fordító ezt és az előző függvényt?
	 *  
	 *  Példa:
	 *  Vektor v(3, 1.0);	// v = {1.0, 1.0, 1.0}
	 *  v[1] = 2.5			// v = {1.0, 2.5, 1.0}
	 */
	double &operator[](size_t idx);
	
	/**
	 *  @brief vektorok összegzése
	 *  @param[in] v jobboldali operandus
	 */
	Vector operator+(Vector const& v) const;

}; // vektor osztály vége

/**
 *  @brief inserter operátor
 *  @param[in] os kimeneti stream
 *  @param[in] v Vektor, amit kiírunk a streamre
 */
std::ostream &operator<<(std::ostream &os, Vector const &v);

#endif 